@@ -15,14 +15,14 @@ import urllib.parse
 import urllib.request
 import warnings
 from collections import defaultdict
-from typing import List, Tuple, Union
+from typing import List, Tuple, Union, Any
 
 import asyncio_atexit
 
 from .. import cdp
 from . import tab, util
 from ._contradict import ContraDict
-from .config import Config, PathLike, is_posix
+from .config import BrowserType, Config, PathLike, is_posix
 from .connection import Connection
 
 logger = logging.getLogger(__name__)
@@ -55,7 +55,7 @@ class Browser:
 
     """
 
-    _process: subprocess.Popen | None
+    _process: subprocess.Popen[bytes] | None
     _process_pid: int | None
     _http: HTTPApi | None = None
     _cookies: CookieJar | None = None
@@ -73,12 +73,13 @@ class Browser:
         headless: bool = False,
         user_agent: str | None = None,
         browser_executable_path: PathLike | None = None,
+        browser: BrowserType = "auto",
         browser_args: List[str] | None = None,
         sandbox: bool = True,
         lang: str | None = None,
         host: str | None = None,
         port: int | None = None,
-        **kwargs,
+        **kwargs: Any,
     ) -> Browser:
         """
         entry point for creating an instance
@@ -89,6 +90,7 @@ class Browser:
                 headless=headless,
                 user_agent=user_agent,
                 browser_executable_path=browser_executable_path,
+                browser=browser,
                 browser_args=browser_args or [],
                 sandbox=sandbox,
                 lang=lang,
@@ -129,7 +131,7 @@ class Browser:
         # use to help manage the browser instance data (needed for multiple browsers)
         self.config = copy.deepcopy(config)
 
-        self.targets: List = []
+        self.targets: List[Connection] = []
         """current targets (all types)"""
         self.info: ContraDict | None = None
         self._target = None
@@ -140,16 +142,21 @@ class Browser:
         logger.debug("Session object initialized: %s" % vars(self))
 
     @property
-    def websocket_url(self):
+    def websocket_url(self) -> str:
         if not self.info:
             raise RuntimeError("Browser not yet started. use await browser.start()")
 
-        return self.info.webSocketDebuggerUrl
+        return self.info.webSocketDebuggerUrl  # type: ignore
 
     @property
-    def main_tab(self) -> tab.Tab:
+    def main_tab(self) -> tab.Tab | None:
         """returns the target which was launched with the browser"""
-        return sorted(self.targets, key=lambda x: x.type_ == "page", reverse=True)[0]
+        results = sorted(self.targets, key=lambda x: x.type_ == "page", reverse=True)
+        if len(results) > 0:
+            result = results[0]
+            if isinstance(result, tab.Tab):
+                return result
+        return None
 
     @property
     def tabs(self) -> List[tab.Tab]:
@@ -157,7 +164,7 @@ class Browser:
         :return:
         """
         tabs = filter(lambda item: item.type_ == "page", self.targets)
-        return list(tabs)
+        return list(tabs)  # type: ignore
 
     @property
     def cookies(self) -> CookieJar:
@@ -166,11 +173,8 @@ class Browser:
         return self._cookies
 
     @property
-    def stopped(self):
-        if self._process and self._process.returncode is None:
-            return False
-        return True
-        # return (self._process and self._process.returncode) or False
+    def stopped(self) -> bool:
+        return not (self._process and self._process.poll() is None)
 
     async def wait(self, time: Union[float, int] = 1) -> Browser:
         """wait for <time> seconds. important to use, especially in between page navigation
@@ -191,7 +195,7 @@ class Browser:
             cdp.target.TargetCreated,
             cdp.target.TargetCrashed,
         ],
-    ):
+    ) -> None:
         """this is an internal handler which updates the targets when chrome emits the corresponding event"""
 
         async with self._update_target_info_mutex:
@@ -217,7 +221,7 @@ class Browser:
                         % (self.targets.index(current_tab), changes_string)
                     )
 
-                    current_tab.target = target_info
+                current_tab.target = target_info
 
             elif isinstance(event, cdp.target.TargetCreated):
                 target_info = event.target_info
@@ -248,7 +252,7 @@ class Browser:
                 self.targets.remove(current_tab)
 
     async def get(
-        self, url="about:blank", new_tab: bool = False, new_window: bool = False
+        self, url: str = "about:blank", new_tab: bool = False, new_window: bool = False
     ) -> tab.Tab:
         """top level get. utilizes the first tab to retrieve given url.
 
@@ -293,11 +297,11 @@ class Browser:
                     lambda item: item.type_ == "page" and item.target_id == target_id,
                     self.targets,
                 )
-            )
+            )  # type: ignore
             connection.browser = self
         else:
             # first tab from browser.tabs
-            connection = next(filter(lambda item: item.type_ == "page", self.targets))
+            connection = next(filter(lambda item: item.type_ == "page", self.targets))  # type: ignore
             # use the tab to navigate to new url
             await connection.send(cdp.page.navigate(url))
             connection.browser = self
@@ -442,7 +446,7 @@ class Browser:
             logger.debug("Could not start", exc_info=True)
             return False
 
-    async def grant_all_permissions(self):
+    async def grant_all_permissions(self) -> None:
         """
         grant permissions for:
             accessibilityEvents
@@ -479,7 +483,9 @@ class Browser:
         permissions.remove(cdp.browser.PermissionType.CAPTURED_SURFACE_CONTROL)
         await self.connection.send(cdp.browser.grant_permissions(permissions))
 
-    async def tile_windows(self, windows=None, max_columns: int = 0):
+    async def tile_windows(
+        self, windows: List[tab.Tab] | None = None, max_columns: int = 0
+    ) -> List[List[int]]:
         import math
 
         import mss
@@ -492,7 +498,7 @@ class Browser:
             screen_height = screen["height"]
         if not screen or not screen_width or not screen_height:
             warnings.warn("no monitors detected")
-            return
+            return []
         await self.update_targets()
         distinct_windows = defaultdict(list)
 
@@ -543,13 +549,12 @@ class Browser:
         info = await self.connection.send(cdp.target.get_targets(), _is_update=True)
         return info
 
-    async def update_targets(self):
+    async def update_targets(self) -> None:
         targets: List[cdp.target.TargetInfo]
         targets = await self._get_targets()
         for t in targets:
             for existing_tab in self.targets:
-                existing_target = existing_tab.target
-                if existing_target.target_id == t.target_id:
+                if existing_tab.target_id == t.target_id:
                     existing_tab.target.__dict__.update(t.__dict__)
                     break
             else:
@@ -567,21 +572,26 @@ class Browser:
 
         await asyncio.sleep(0)
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> Browser:
         return self
 
-    async def __aexit__(self, exc_type, exc_val, exc_tb):
+    async def __aexit__(
+        self, exc_type: type[BaseException] | None, exc_val: Any, exc_tb: Any
+    ) -> None:
         if exc_type and exc_val:
             raise exc_type(exc_val)
 
-    def __iter__(self):
-        self._i = self.tabs.index(self.main_tab)
+    def __iter__(self) -> Browser:
+        main_tab = self.main_tab
+        if not main_tab:
+            return self
+        self._i = self.tabs.index(main_tab)
         return self
 
-    def __reversed__(self):
-        return reversed(list(self.tabs))
+    def __reversed__(self) -> List[tab.Tab]:
+        return list(reversed(list(self.tabs)))
 
-    def __next__(self):
+    def __next__(self) -> tab.Tab:
         try:
             return self.tabs[self._i]
         except IndexError:
@@ -597,11 +607,17 @@ class Browser:
                 else:
                     del self._i
 
-    async def stop(self):
+    async def stop(self) -> None:
         if not self.connection and not self._process:
             return
 
-        if self.connection:
+        if self.connection and not self.connection.closed:
+            try:
+                await self.connection.send(cdp.browser.close())
+            except Exception:
+                logger.warning(
+                    "Could not send the close command when stopping the browser. Likely the browser is already gone. Closing the connection."
+                )
             await self.connection.aclose()
             logger.debug("closed the connection")
 
@@ -653,7 +669,7 @@ class Browser:
                 await asyncio.sleep(0.15)
                 continue
 
-    def __del__(self):
+    def __del__(self) -> None:
         pass
 
 
@@ -690,7 +706,7 @@ class CookieJar:
             import requests.cookies
 
             return [
-                requests.cookies.create_cookie(
+                requests.cookies.create_cookie(  # type: ignore
                     name=c.name,
                     value=c.value,
                     domain=c.domain,
@@ -702,7 +718,7 @@ class CookieJar:
             ]
         return cookies
 
-    async def set_all(self, cookies: List[cdp.network.CookieParam]):
+    async def set_all(self, cookies: List[cdp.network.CookieParam]) -> None:
         """
         set cookies
 
@@ -724,7 +740,7 @@ class CookieJar:
 
         await connection.send(cdp.storage.set_cookies(cookies))
 
-    async def save(self, file: PathLike = ".session.dat", pattern: str = ".*"):
+    async def save(self, file: PathLike = ".session.dat", pattern: str = ".*") -> None:
         """
         save all cookies (or a subset, controlled by `pattern`) to a file to be restored later
 
@@ -778,7 +794,7 @@ class CookieJar:
                 break
         pickle.dump(cookies, save_path.open("w+b"))
 
-    async def load(self, file: PathLike = ".session.dat", pattern: str = ".*"):
+    async def load(self, file: PathLike = ".session.dat", pattern: str = ".*") -> None:
         """
         load all cookies (or a subset, controlled by `pattern`) from a file created by :py:meth:`~save_cookies`.
 
@@ -814,7 +830,7 @@ class CookieJar:
                 break
         await self.set_all(included_cookies)
 
-    async def clear(self):
+    async def clear(self) -> None:
         """
         clear current cookies
 
@@ -842,13 +858,15 @@ class HTTPApi:
         self.host, self.port = addr
         self.api = "http://%s:%d" % (self.host, self.port)
 
-    async def get(self, endpoint: str):
+    async def get(self, endpoint: str) -> Any:
         return await self._request(endpoint)
 
-    async def post(self, endpoint, data):
-        return await self._request(endpoint, data)
+    async def post(self, endpoint: str, data: dict[str, str]) -> Any:
+        return await self._request(endpoint, method="post", data=data)
 
-    async def _request(self, endpoint, method: str = "get", data: dict | None = None):
+    async def _request(
+        self, endpoint: str, method: str = "get", data: dict[str, str] | None = None
+    ) -> Any:
         url = urllib.parse.urljoin(
             self.api, f"json/{endpoint}" if endpoint else "/json"
         )
